@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -
 #
-# Copyright (c) 2008 (c) Benoit Chesneau <benoitc@e-engura.com> 
+# Copyright (c) 2008, 2009 Benoit Chesneau <benoitc@e-engura.com> 
 #
 # Permission to use, copy, modify, and distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -216,7 +216,7 @@ class Resource(object):
         """
         to set a new uri absolute path
         """
-        self.uri = self.client.transport.make_uri(self.uri, path)
+        self.uri = self.client.make_uri(self.uri, path)
 
 
 class RestClient(object):
@@ -376,7 +376,7 @@ class RestClient(object):
         # build the path
         path = "/".join([''] +
                         [url_quote(s.strip('/'), self.charset, self.safe) for s in path
-                         if s is not None])
+                         if s is not None and isinstance(s, basestring)])
 
         if path:
             retval.append(path)
