@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -
 #
-# Copyright (c) 2008 (c) Benoit Chesneau <benoitc@e-engura.com> 
+# Copyright (c) 2008, 2009 Benoit Chesneau <benoitc@e-engura.com> 
 #
 # Permission to use, copy, modify, and distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -16,10 +16,12 @@
 #
 import StringIO
 import httplib
+
 import re
 import sys
 
 import restclient
+from restclient.iri2uri import iri2uri
 
 try:
     import httplib2
@@ -40,6 +42,10 @@ class TransportError(Exception):
 USER_AGENT = "py-restclient/%s (%s)" % (restclient.__version__, sys.platform)
 DEFAULT_MAX_REDIRECT = 3
 
+NORMALIZE_SPACE = re.compile(r'(?:\r\n)?[ \t]+')
+def _normalize_headers(headers):
+    return dict([ (key.lower(), NORMALIZE_SPACE.sub(value, ' ').strip())  for (key, value) in headers.iteritems()])
+
 def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
     """
     Returns a bytestring version of 's', encoded as specified in 'encoding'.
@@ -67,7 +73,6 @@ def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
     else:
         return s
 
-
 def createHTTPTransport():
     """Create default HTTP client instance
     prefers Curl to urllib"""
@@ -85,7 +90,6 @@ def getDefaultHTTPTransport():
 
     :return: the default client
     """
-
     global _default_http
 
     if _default_http is None:
@@ -132,6 +136,23 @@ class HTTPResponse(dict):
 class HTTPTransportBase(object):
     """ Interface for HTTP clients """
 
+    def __init__(self, proxy_infos=None):
+        """ constructor for HTTP transport interface
+
+        :param proxy_infos: dict, infos to connect via proxy:
+
+        .. code-block:: python
+
+            {
+                'proxy_user': 'XXXXXXX',
+                'proxy_password': 'XXXXXXX',
+                'proxy_host': 'proxy',
+                'proxy_port': 8080,
+            }
+        """
+        self._credentials = {}
+        self.proxy_infos = proxy_infos or {}
+
     def request(self, url, method='GET', body=None, headers=None):
         """Perform HTTP call and manage , support GET, HEAD, POST, PUT and
         DELETE
@@ -145,6 +166,16 @@ class HTTPTransportBase(object):
         """
         raise NotImplementedError
 
+    def add_credentials(self, user, password):
+        self._credentials = {
+                "user": user,
+                "password": password
+        }
+
+    def _get_credentials(self):
+        return self._credentials
+
+
 def _get_pycurl_errcode(symbol, default):
     """
     Returns the numerical error code for a symbol defined by pycurl.
@@ -196,6 +227,8 @@ class CurlTransport(HTTPTransportBase):
         :param timeout: int, timeout of request
         :param proxy_infos: dict, infos to connect via proxy:
 
+        .. code-block:: python
+
             {
                 'proxy_user': 'XXXXXXX',
                 'proxy_password': 'XXXXXXX',
@@ -203,9 +236,7 @@ class CurlTransport(HTTPTransportBase):
                 'proxy_port': 8080,
             }
         """
-        
-        HTTPTransportBase.__init__(self)
-        self._credentials = {}
+        HTTPTransportBase.__init__(self, proxy_infos=proxy_infos)
 
         # path to certificate file
         self.cabundle = None
@@ -214,33 +245,46 @@ class CurlTransport(HTTPTransportBase):
             raise RuntimeError('Cannot find pycurl library')
 
         self.timeout = timeout
-        self.proxy_infos = proxy_infos or {}
-            
+        
+    def _parseHeaders(self, header_file):
+        header_file.seek(0)
+       
+        # Remove the status line from the beginning of the input
+        unused_http_status_line = header_file.readline()
+        lines = [line.strip() for line in header_file]
+        
+        # and the blank line from the end
+        empty_line = lines.pop()
+        if empty_line:
+            raise TransportError("No blank line at end")
+       
+        headers = {}
+        for line in lines:
+            if ":" in line:
+                try:
+                    name, value = line.split(':', 1)
+                except ValueError:
+                    raise TransportError(
+                        "Malformed HTTP header line in response: %r" % (line,))
 
-    def _parseHeaders(self, status_and_headers):
-        status_and_headers.seek(0)
+                value = value.strip()
+
+                # HTTP headers are case-insensitive
+                name = name.lower()
+                headers[name] = value
+
+        return headers
 
-        # Ignore status line
-        status_and_headers.readline()
-        msg = httplib.HTTPMessage(status_and_headers)
-        return dict(msg.items())
 
     def request(self, url, method='GET', body=None, headers=None):
-        put = method in ('PUT')
         body = body or ""        
         headers = headers or {}
         headers.setdefault('User-Agent',
                            "%s %s" % (USER_AGENT, pycurl.version,))
 
-        # turn off default pragma provided by Curl
-        headers.update({
-            'Cache-control': 'max-age=0',
-            'Pragma': 'no-cache'
-        })
-
-        if put:
-            headers.setdefault('Expect', '100-continue')
-
+        # encode url
+        url = iri2uri(url)
+        
         c = pycurl.Curl()
         try:
             # set curl options
@@ -253,9 +297,10 @@ class CurlTransport(HTTPTransportBase):
             header = StringIO.StringIO()
             c.setopt(pycurl.WRITEFUNCTION, data.write)
             c.setopt(pycurl.HEADERFUNCTION, header.write)
-            c.setopt(pycurl.URL , smart_str(url))
+            c.setopt(pycurl.URL, smart_str(url))
             c.setopt(pycurl.FOLLOWLOCATION, 1)
             c.setopt(pycurl.MAXREDIRS, 5)
+            c.setopt(pycurl.NOSIGNAL, 1)
 
             if self.cabundle:
                 c.setopt(pycurl.CAINFO, celf.cabundle)
@@ -309,22 +354,25 @@ class CurlTransport(HTTPTransportBase):
                         del headers['Content-Length']
                     content_length = len(body)
 
-                if put:
-                    c.setopt(pycurl.INFILESIZE, content_length)
                 if method in ('POST'):
                     c.setopt(pycurl.POSTFIELDSIZE, content_length)
+                else:
+                    c.setopt(pycurl.INFILESIZE, content_length)
                 c.setopt(pycurl.READFUNCTION, content.read)
             
             if headers:
+                _normalize_headers(headers)
+
                 c.setopt(pycurl.HTTPHEADER,
                         ["%s: %s" % pair for pair in sorted(headers.iteritems())])
 
+                
             try:
                 c.perform()
             except pycurl.error, e:
                 if e[0] != CURLE_SEND_ERROR:
                     raise TransportError(e)
-
+ 
             response_headers = self._parseHeaders(header)
             code = c.getinfo(pycurl.RESPONSE_CODE)
             
@@ -333,16 +381,6 @@ class CurlTransport(HTTPTransportBase):
         finally:
             c.close()
 
-    def add_credentials(self, user, password):
-        self._credentials = {
-                "user": user,
-                "password": password
-        }
-
-    def _get_credentials(self):
-        return self._credentials
-
-
     def _make_response(self, final_url=None, status=None, headers=None,
             body=None):
         resp = HTTPResponse()
@@ -361,19 +399,55 @@ class HTTPLib2Transport(HTTPTransportBase):
         `Httplib2 <http://code.google.com/p/httplib2/>`_
     """
 
-    def __init__(self, http=None):
-        """@param http: An httplib2.HTTP instance.
+    def __init__(self, proxy_infos=None, http=None):
+        """
+        :param proxy_infos: dict, infos to connect via proxy:
+
+        .. code-block:: python
+    
+            {
+                'proxy_user': 'XXXXXXX',
+                'proxy_password': 'XXXXXXX',
+                'proxy_host': 'proxy',
+                'proxy_port': 8080,
+            }
+
+        :param http: An httplib2.HTTP instance.
+
+
         """
         if httplib2 is None:
             raise RuntimeError('Cannot find httplib2 library. '
                                'See http://bitworking.org/projects/httplib2/')
 
-        super(HTTPLib2Transport, self).__init__()
+        super(HTTPLib2Transport, self).__init__(proxy_infos=proxy_infos)
         
-        if http is None:
-            http = httplib2.Http()
+        _proxy_infos = None
+        if proxy_infos and proxy_infos is not None:
+            try:
+                import socks
+            except:
+                print >>sys.stderr, "socks module isn't installed, you can't use proxy"
+                socks = None
 
+            if socks is not None:
+                _proxy_infos = httplib2.ProxyInfo(
+                        socks.PROXY_TYPE_HTTP,
+                        proxy_infos.get('proxy_host'),
+                        proxy_infos.get('proxy_port'),
+                        proxy_infos.get('proxy_username'),
+                        proxy_infos.get('proxy_password')
+                )
+
+        if http is None:
+            http = httplib2.Http(proxy_info=_proxy_infos)
+        else:
+            if _proxy_infos is not None and \
+                    not http.proxy_info and \
+                    http.proxy_info is None:
+                proxy_info = _proxy_infos
         self.http = http
+        
         self.http.force_exception_to_status_code = False
 
     def request(self, url, method='GET', body=None, headers=None):
@@ -402,7 +476,6 @@ class HTTPLib2Transport(HTTPTransportBase):
         httplib2_response, content = self.http.request(url,
                 method=method, body=content, headers=headers)
 
-
         try:
             final_url = httplib2_response['content-location']
         except KeyError:
@@ -415,3 +488,8 @@ class HTTPLib2Transport(HTTPTransportBase):
         resp.body = content
 
         return resp, content
+
+    def add_credentials(self, user, password):
+        super(HTTPLib2Transport, self).add_credentials(user, password)
+        self.http.add_credentials(user, password)
+
